% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Part II:		Ontology information									% %
% %																				% %
% %				All the predicates to get|set information in the ontology		% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Axiom Internal Representation											% %
% %																				% %
% %		Each axiom is stored using the Prolog (dynamic) predicate "axiom/6":	% %
% %																				% %
% %		axiom(Code,File,FirstLine,LastLine,AxiomKey,FOFormula)					% %
% %																				% %
% %			Code: An internal code about the axiom								% %
% %				-3 -> Documentation												% %
% %				-2 -> Empty line												% %
% %				-1 -> Comment													% %
% %				 0 -> FO axiom													% %
% %				 1 -> FO axiom with rowList										% %
% %				 2 -> FO axiom with rowOperator									% %
% %				 3 -> Domain													% %
% %				 4 -> Schema													% %
% %				 5 -> Axiom with functions or non-FO axiom						% %
% %				  -> ...														% %
% %																				% %
% %			File: name of file of the axiom										% %
% %																				% %
% %			FirstLine: First line of the axiom in the file						% %
% %																				% %
% %			LastLine: Last line of the axiom in the file						% %
% %																				% %
% %			AxiomKey: an atom starting fromt the name of the file				% %
% %							and followed by a number							% %
% %						comments and empty lines have the key "text"			% %
% %						documentations have the key "documentation"				% %
% %																				% %
% %			FO formulas are represented using the following functions:			% %
% %																				% %
% %					Exists (typed) ->		exists(LV,F)						% %
% %					For all (typed) ->		forall(LV,F)						% %
% %					Exists (non-typed) ->	exists*(LV,F)						% %
% %					For all (non-typed) ->	forall*(LV,F)						% %
% %					Conjunction ->			and(LF)								% %
% %					Disjunction ->			or(LF)								% %
% %					Row conjunction ->		@and(RV,H,S,F)						% %
% %					Row disjunction ->		@or(RV,H,S,F)						% %
% %					Implication ->			implies(F1,F2)						% %
% %					Equivalence ->			equiv(F1,F2)						% %
% %					Negation ->				not(F)								% %
% %					Equality ->				equal(T1,T2)						% %
% %																				% %
% %				where	LV is a list of Variables								% %
% %							(a Variable is a Prolog atom starting by '_'		% %
% %								or uppercase)									% %
% %						F, F1, F2 are formulas									% %
% %						LF is a list of formulas								% %
% %						T1, T2 are terms										% %
% %						RV is a Row variable									% %
% %						H is the head of a Row variable (a Variable)			% %
% %						S is the tail of a Row variable							% %
% %							(a Row variable)									% %
% %																				% %
% %			Terms are represented as Prolog terms								% %
% %																				% %
% %			Variables are represented using "'#var'/1"							% %
% %																				% %
% %			Row variables are represented using "'#rowVar'/1"					% %
% %																				% %
% %			Multi-length lists' instances are represented using "'#row'/1"		% %
% %																				% %
% %			Meta-information is represented using the following functions:		% %
% %																				% %
% %				Schema ->	$schema(C,AUXR,I,VP,F)								% %
% %				Domain ->	$domain(R,I,C)										% %
% %																				% %
% %				where	C is a class											% %
% %						AUXR,R,are relations									% %
% %						VP is a variable predicate								% %
% %						F is a formula											% %
% %						I is an index											% %
% %																				% %
% %		Besides, the Prolog (dynamic) predicate "numberOfLines/1" stores		% %
% %			the number of lines in each file and "numberOfAxioms/2" stores		% %
% %			the number of axioms in each file									% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic	axiom/6, numberOfLines/1, numberOfAxioms/2. 
:- dynamic	goal/5, numberOfGoals/2, goalInformation/5.

%%% saveAxiom(in C, in FILE, in FL, in LL, in K, in F)
%%%		C: axiom type
%%%		FILE: axiom file
%%%		FL: first line of the axiom in FILE
%%%		LL: last line of the axiom in FILE
%%%		K: axiom key
%%%		F: axiom formula
%%%		Action: the axiom is stored

saveAxiom(C,FILE,FL,LL,K,F):-
		(
		 (
		  retract((axiom(C,FILE,FL,LL,K,F))),
		  !
		 );
		 (
		  true
		 )
		),
		assertz((axiom(C,FILE,FL,LL,K,F))).

%%% getAxiom(in K, out C, out F)
%%%		K: axiom key
%%%		C: axiom type
%%%		F: axiom formula

getAxiom(K,C,F):-
		axiom(C,_,_,_,K,F).

%%% getCompleteAxiom(in K, out C, out FILE, out FL, out LL, out F)
%%%		K: axiom key
%%%		C: axiom type
%%%		FILE: axiom file
%%%		FL: first line of the axiom in FILE
%%%		LL: last line of the axiom in FILE
%%%		F: axiom formula

getCompleteAxiom(K,C,FILE,FL,LL,F):-
		axiom(C,FILE,FL,LL,K,F).

%%% getAxiomByFileLine(in FILE, in FL, out, C, out LL, out F)
%%%		FILE: axiom file
%%%		FL: first line of the axiom in FILE
%%%		C: axiom type
%%%		LL: last line of the axiom in FILE
%%%		K: axiom key
%%%		F: axiom formula

getAxiomByFileLine(FILE,FL,C,LL,K,F):-
		axiom(C,FILE,FL,LL,K,F).

%%% setAxiom(in K, in C, in F)
%%%		K: axiom key
%%%		C: axiom type
%%%		F: axiom formula
%%%		Action: the axiom of key K is update with C and F

setAxiom(K,C,F):-
		axiom(_,FILE,FL,LL,K,_),
		retract((axiom(_,_,_,_,K,_))),
		assertz((axiom(C,FILE,FL,LL,K,F))).

%%% getNumberOfAxioms(in F, out N)
%%%		F: file
%%%		N: number of axioms in F

getNumberOfAxioms(F,N):-
		numberOfAxioms(F,N).

%%% setNumberOfAxioms(in F, out N)
%%%		F: file
%%%		N: number of axioms in F
%%%		Action: the number of axioms in F is saved

setNumberOfAxioms(F,N):-
		retractall((numberOfAxioms(F,_))),
		assertz((numberOfAxioms(F,N))).

%%% getNumberOfLines(in F, out N)
%%%		F: file
%%%		N: number of text lines in F

getNumberOfLines(F,N):-
		numberOfLines(F,N).

%%% setNumberOfLines(in F, out N)
%%%		F: file
%%%		N: number of text lines in F
%%%		Action: the number of text lines in F is saved

setNumberOfLines(F,N):-
		retractall((numberOfLines(F,_))),
		assertz((numberOfLines(F,N))).

%%% saveGoal(in FILE, in FL, in LL, in K, in F)
%%%		FILE: goal file
%%%		FL: first line of the goal in FILE
%%%		LL: last line of the goal in FILE
%%%		K: goal key
%%%		F: goal formula
%%%		Action: the goal is stored

saveGoal(FILE,FL,LL,K,F):-
		(
		 (
		  retract((goal(FILE,FL,LL,K,F))),
		  !
		 );
		 (
		  true
		 )
		),
		assertz((goal(FILE,FL,LL,K,F))).

%%% getGoal(in K, out F) but also used as (out K, in F)
%%%		K: goal key
%%%		F: goal formula

getGoal(K,F):-
		goal(_,_,_,K,F).

%%% setGoal(in K,in F)
%%%		K: goal key
%%%		F: goal formula
%%%		Action: the goal of key K is updated with F

setGoal(K,F):-
		goal(FILE,FL,LL,K,_),
		retract((goal(_,_,_,K,_))),
		assertz((goal(FILE,FL,LL,K,F))).

%%% getGoalInformation(in K, out LI)
%%%		K: goal key
%%%		LI: list of information of the goal, where each element is of the form (R,M1,M2,C)
%%%			R: role
%%%			M1,M2: mappings to AdimenSUMO
%%%			MT1,MT2: type of mappings to AdimenSUMO
%%%			C: code of the action

getGoalInformation(K,LI):-
		findall((R,M1,M2,C),goalInformation(K,R,M1,M2,C),LI).

%%% setGoalInformation(in K, in R, in M1, in M2, in C)
%%%		K: goal key
%%%		R: role
%%%		M1,M2: mappings to AdimenSUMO
%%%		C: code of the action
%%%		Action: the information of goal K is updated with K

setGoalInformation(K,R,M1,M2,C):-
		assertz((goalInformation(K,R,M1,M2,C))).

%%% getNumberOfGoals(in F, out N)
%%%		F: file
%%%		N: number of goals in F

getNumberOfGoals(F,N):-
		numberOfGoals(F,N).

%%% setNumberOfGoals(in F, out N)
%%%		F: file
%%%		N: number of goals in F
%%%		Action: the number of goals in F is saved

setNumberOfGoals(F,N):-
		retractall((numberOfGoals(F,_))),
		assertz((numberOfGoals(F,N))).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Index of auxiliary variables:											% %
% %																				% %
% %			We use the function "index_of_auxiliary_variables/1"				% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic index_of_auxiliary_variables/1.

%%% indexOfAuxiliaryVariables(out I)
%%%		I: current index of auxiliary variables

indexOfAuxiliaryVariables(I):-
		index_of_auxiliary_variables(I),
		NI is I + 1,
		retractall((index_of_auxiliary_variables(I))),
		assertz((index_of_auxiliary_variables(NI))).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %	Predicate and Constant information											% %
% %																				% %
% %		Relations are stored using the following (dynamic) predicate:			% %
% %																				% %
% %			inputPredicate(P,A)													% %
% %																				% %
% %			where	P is a predicate name										% %
% %						A is the arity of P										% %
% %																				% %
% %		Constants are stored using the following (dynamic) predicate:			% %
% %																				% %
% %			inputConstant(C)													% %
% %																				% %
% %			where	C is a constant name										% %
% %																				% %
% %		Further, we store the arity of row variables for each predicate			% %
% %																				% %
% %			rowArity(P,I,A)														% %
% %																				% %
% %			where	P is a predicate											% %
% %						I is the argument of the row variable					% %
% %						A is the arity of the row variable						% %
% %																				% %
% %		Finally, some arguments of predicates are related, if the same			% %
% %			row variable occurs in those arguments								% %
% %																				% %
% %			relatedArity(P1,I1,P2,I2)											% %
% %																				% %
% %			where	P1, P2 are predicates										% %
% %						I1, I2 are the arguments								% %
% %																				% %
% %		In the same way, we relate the arguments where tail row variables in	% %
% %			row operators occurs with the arguments where row variables			% %
% %																				% %
% %			relatedTailArity(P1,I1,P2,I2)										% %
% %																				% %
% %			where	P1, P2 are predicates										% %
% %						I1, I2 are the arguments								% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic	inputPredicate/2, inputConstant/1, rowArity/3,
		relatedArity/4, relatedTailArity/4.

%%% addInputConstant(in C)
%%%		C: a constant
%%%		Action: constant C is added

%addInputConstant('not'):-
%		!,
%		trace.
addInputConstant(C):-
		inputConstant(C),
		!.
addInputConstant(C):-
		assertz((inputConstant(C))).

%%% addInputPredicate(in P, in A)
%%%		P: predicate name
%%%		A: predicate arity
%%%		Action: predicate "P/A" is added

%addInputPredicate('ProbablityFn',_):-
%		!,
%		trace.
addInputPredicate(P,A):-
		inputPredicate(P,A),
		!.
addInputPredicate(P,A):-
		assertz((inputPredicate(P,A))).

%%% getInputConstant(in-out C)
%%%		C: a constant
%%%		Action:
%%%			success		if C IS a constant
%%%			fail		if c is NOT a constant

getInputConstant(C):-
		inputConstant(C),
		!.

%%% getPredicate(in-out P, in-out A)
%%%		P: predicate name
%%%		A: predicate arity
%%%		Action:
%%%			success		if "P/A" IS a predicate
%%%			fail		if "P/A" is NOT a predicate

getInputPredicate(P,A):-
		inputPredicate(P,A),
		!.

%%% getAllInputConstants(out LC)
%%%		LC: the list of all constants defined in the ontology

getAllInputConstants(LC):-
		findall(C,inputConstant(C),LC).

%%% getAllInputPredicates(out LP)
%%%		LP: the list of all predicates represented as pairs (S,A)
%%%			where
%%%				S:	predicate symbol
%%%				A:	predicate arity

getAllInputPredicates(LP):-
		findall((S,A),inputPredicate(S,A),LP).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		List of ontologies:														% %
% %																				% %
% %			We use the (dynamic) predicate "ontologyList/1"						% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic	ontologyList/1, adimenSUMOList/1, goalList/1.

:- retractall((ontologyList(_))).

%:- assertz((ontologyList([predefinitions,merge195,milo1133]))).
%:- assertz((ontologyList([predefinitions,merge195]))).

:- assertz((ontologyList([
		predefinitions,
		merge,
		milo,
		arabicCulture,
		cars,
		catalog,
		cctrep,
		communications,
		computerInput,
		countriesAndRegions,
		dining,
		economy,
		emotion,
		engineering,
		financialOntology,
		food,
		geography,
		government,
		hotel,
		justice,
		languages,
		media,
		military,
		militaryDevices,
		militaryPersons,
		militaryProcesses,
		mondial,
		music,
		naics,
		people,
		qoSontology,
		sports,
		transnationalIssues,
		transportation,
		transportDetail,
		uxexperimentalTerms,
		virusProteinAndCellPart,
		wmd,
		worldAirports
]))).

:- assertz((adimenSUMOList([
		predefinitions,
		merge,
		milo
]))).

:- retractall((goalList(_))).

%:- assertz((goalList([goal00,goal01,goal10]))).
:- assertz((goalList([goal]))).


%%% getOntologyList(out L)
%%%		L: list of ontologies

getOntologyList(L):-
		ontologyList(L).

%%% getAdimenSUMOList(out L)
%%%		L: list of ontologies

getAdimenSUMOList(L):-
		adimenSUMOList(L).

%%% setOntologyList(in L)
%%%		L: list of ontologies
%%%		Action: the ontology list is updated to L

setOntologyList(L):-
		retractall((ontologyList(_))),
		assertz((ontologyList(L))).

%%% setAdimenSUMOList(in L)
%%%		L: list of ontologies
%%%		Action: the AdimenSUMO list is updated to L

setAdimenSUMOList(L):-
		retractall((adimenSUMOList(_))),
		assertz((adimenSUMOList(L))).

%%% getGoalList(out L)
%%%		L: list of goals

getGoalList(L):-
		goalList(L).

%%% setGoalList(in L)
%%%		L: list of goals
%%%		Action: the goal list is updated to L

setGoalList(L):-
		retractall((goalList(_))),
		assertz((goalList(L))).

%%% getGoalKey(in F, in C, out K)
%%%		F: name of the file
%%%		C: index of goals
%%%		K: key of the goal

getGoalKey(F,C,K):-
		number_codes(C,LC),
		extendToThousand(LC,NLC),
		atom_codes(NC,NLC),
		atom_concat(F,NC,K).

%%% getNegatedGoalSubkey(in K, out NK)
%%%		K: subkey of goal
%%%		NK: subkey of negated goal

getNegatedGoalSubkey(K,NK):-
		atom_codes(K,[C|LC]),
		char_code('A',C1),
		char_code('a',C2),
		NC is C + C1 - C2,
		atom_codes(SK,[NC|LC]),		
		atom_concat('negated',SK,NK).

%%% getNegatedGoalKey(in K, out NK)
%%%		K: goal key
%%%		NK: negated goal key

getNegatedGoalKey(K,NK):-
		getNegatedGoalSubkey(K,NK).

%%% extendToThousand(in LC, out NLC)
%%%		LC: list of characters that represents a number
%%%		NLC: list of characters that represents the same number with at least 4 digits

extendToThousand(LC,LC):-
		length(LC,N),
		N >= 4,
		!.
extendToThousand(LC,NLC):-
		atom_codes('0',[C]),
		extendToThousand([C|LC],NLC).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Delete previous information:											% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

deletePreviousOntologyInformation:-
		retractall((axiom(_,_,_,_,_,_))),										% Delete previous information
		retractall((goal(_,_,_,_,_))),
		retractall((inputPredicate(_,_))),
		retractall((inputConstant(_))),
		retractall((rowArity(_,_,_))),
		retractall((relatedArity(_,_,_,_))),
		retractall((relatedTailArity(_,_,_,_))),
		retractall((numberOfAxioms(_,_))),
		retractall((numberOfLines(_,_))),
		retractall((numberOfGoals(_,_))),
		retractall((goalInformation(_,_,_,_,_,_,_))),
		retractall((index_of_auxiliary_variables(_))),
		retractall(('&topClass'(_))),
		retractall(('&instanceOf'(_,_))),
		retractall(('&subclassOf'(_,_))),
		retractall(('&ontologyOf'(_,_))),
		retractall(('&initializedClass'(_))),
		retractall(('&initializedObject'(_))),
		retractall(('&initializedSubrelation'(_))),
		retractall(('&initializedSubAttribute'(_))),
		retractall(('&disjoint'(_,_))),
%		retractall(('&disjointDecomposition'(_,_))),
%		retractall(('&partition'(_,_))),
		retractall(('&subrelationOf'(_,_))),
		retractall(('&subAttributeOf'(_,_))).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Initialize auxiliary information:										% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


% % All the definitions and information that will be used by the compiler

% % Dynamic Predicates:
% %			- '&topClass'/1
% %			- '&instanceOf'/2
% %			- '&subclassOf'/2
% %			- '&ontologyOf'/2
% %			- '&disjoint'/2
% %			- '&disjointDecomposition'/2
% %			- '&partition'/2
% %			- '&subrelationOf'/2
% %			- '&subAttributeOf'/2

% %			- '&mapping2AdimenSUMO'/2

:- dynamic	'&topClass'/1, '&instanceOf'/2, '&subclassOf'/2, '&ontologyOf'/2,
		'&initializedClass'/1, '&initializedObject'/1,
		'&initializedSubrelation'/1, '&initializedSubAttribute'/1,
		'&disjoint'/2, %'&disjointDecomposition'/2,'&partition'/2,
		'&subrelationOf'/2, '&subAttributeOf'/2.


%%% initializeOntologyInformation
%%%		Action: internal information is stored using dynamic predicates

initializeOntologyInformation:-
		initializeClassesAndObjects.

%%% initializeClassesAndObjects
%%%		Action: the hierarchy of the ontology is initialized

initializeClassesAndObjects:-
		findall((F,C),axiom(_,F,_,_,_,'$topClass'(C)),LTC),
		initializeListOfTopClasses(LTC).

%%% initializeListOfTopClasses(in LTC)
%%%		LTC: a list of pairs (F,C) where
%%%			F: ontology file where C occurs in
%%%			C: top class
%%%		Action: the hierarchy of the ontology starting from the top class C is initialized

initializeListOfTopClasses([]).
initializeListOfTopClasses([(F,C)|LTC]):-
		assertWithoutRepetition('&topClass'(C),_),
		assertWithoutRepetition('&ontologyOf'(C,F),_),
		initializeClassInformation(C),
		initializeListOfTopClasses(LTC).

%%% initializeClassInformation(in C)
%%%		C: class
%%%		Action: the hierarchy of the ontology starting from the class C is initialized

initializeClassInformation(C):-
		'&initializedClass'(C),
		!.
initializeClassInformation(C):-
		assertWithoutRepetition('&initializedClass'(C),_),
		initializeSubclassesOf(C),
		initializeInstancesOf(C),
		initializeDisjointWith(C),
		initializeDisjointDecompositionsOf(C),	%initializePartitionsOf(C),
		initializeSubrelationsOf(C),											% A class should not have subrelations
		initializeSubAttributesOf(C).											% A class should not have subAttributes

%%% initializeObjectInformation(in O)
%%%		O: object
%%%		Action: the hierarchy of the ontology starting from the object O is initialized

initializeObjectInformation(O):-
		'&initializedObject'(O),
		!.
initializeObjectInformation(O):-
		assertWithoutRepetition('&initializedObject'(O),_),
		initializeSubrelationsOf(O),
		initializeSubAttributesOf(O).

%%% initializeSubrelationInformation(in R)
%%%		R: relation
%%%		Action: the hierarchy of the ontology starting from the relation R is initialized

initializeSubrelationInformation(R):-
		'&initializedSubrelation'(R),
		!.
initializeSubrelationInformation(R):-
		assertWithoutRepetition('&initializedSubrelation'(R),_),
		initializeSubrelationsOf(R).

%%% initializeSubAttributeInformation(in A)
%%%		A: subAttribute
%%%		Action: the hierarchy of the ontology starting from the subAttribute A is initialized

initializeSubAttributeInformation(A):-
		'&initializedSubAttribute'(A),
		!.
initializeSubAttributeInformation(A):-
		assertWithoutRepetition('&initializedSubAttribute'(A),_),
		initializeSubAttributesOf(A).

%%% initializeSubclassesOf(in C)
%%%		C: class
%%%		Action: the hierarchy of the ontology starting from the subclasses of C is initialized

initializeSubclassesOf(C):-
		findall((F,SC),axiom(_,F,_,_,_,'$subclass'(SC,C)),LSC),
		initializeListOfSubclassesOf(C,LSC).
		
%%% initializeListOfSubclassesOf(in C, in LSC)
%%%		C: class
%%%		LSC: a list of pairs (F,SC) where
%%%			F: ontology file where SC occurs in
%%%			SC: subclass of C
%%%		Action: the hierarchy of the ontology starting from the subclass SC of C is initialized

initializeListOfSubclassesOf(_,[]).
initializeListOfSubclassesOf(C,[(F,SC)|LSC]):-
		assertWithoutRepetition('&subclassOf'(SC,C),_),
		assertWithoutRepetition('&ontologyOf'(SC,F),_),
		initializeClassInformation(SC),
		initializeListOfSubclassesOf(C,LSC).

%%% initializeInstancesOf(in C)
%%%		C: class
%%%		Action: the hierarchy of the ontology starting from the instances of C is initialized

initializeInstancesOf(C):-
		findall((F,O),axiom(_,F,_,_,_,'$instance'(O,C)),LO),
		initializeListOfInstancesOf(C,LO).

%%% initializeListOfInstancesOf(in C, in LO)
%%%		C: class
%%%		LO: a list of pairs (F,O) where
%%%			F: ontology file where SC occurs in
%%%			O: instance of C
%%%		Action: the hierarchy of the ontology starting from the instance O of C is initialized

initializeListOfInstancesOf(_,[]).
initializeListOfInstancesOf(C,[(F,O)|LO]):-
		assertWithoutRepetition('&instanceOf'(O,C),_),
		assertWithoutRepetition('&ontologyOf'(O,F),_),
		initializeObjectInformation(O),
		initializeListOfInstancesOf(C,LO).

%%% initializeDisjointWith(in C)
%%%		C: class
%%%		Action: incompability information about C is initialized

initializeDisjointWith(C):-
		findall(DC,axiom(_,_,_,_,_,('$disjoint'(C,DC);'$disjoint'(DC,C))),LDC),
		initializeListOfDisjointClassesWith(C,LDC).

%%% initializeListOfDisjointClassesWith(in C, in LDC)
%%%		C: class
%%%		LDC: a list of pairs (F,DC) where
%%%			F: ontology file where the incompatibility of C and DC is stated
%%%			DC: incompatible class with C
%%%		Action: incompatibility information of C and DC is initialized

initializeListOfDisjointClassesWith(_,[]).
initializeListOfDisjointClassesWith(C,[DC|LDC]):-
		assertWithoutRepetition('&disjoint'(C,DC),_),
		initializeListOfDisjointClassesWith(C,LDC).

%%% initializeDisjointDecompositionsOf(in C)
%%%		C: class
%%%		Action: incompability information about subclasses of C is initialized

initializeDisjointDecompositionsOf(C):-
		findall(DSC,axiom(_,_,_,_,_,('$disjointDecomposition'(C,DSC);'$partition'(C,DSC))),LDSC),
		initializeListOfDisjointDecompositions(LDSC).

%%% initializeListOfDisjointDecompositions(in LDC)
%%%		LDC: a list of 'list of disjoint decompositions'
%%%		Action: incompatibility information is initialized

initializeListOfDisjointDecompositions([]).
initializeListOfDisjointDecompositions([DC|LDC]):-
		initializeDisjointDecomposition(DC),
		initializeListOfDisjointDecompositions(LDC).

%%% initializeDisjointDecomposition(in DC)
%%%		DC: disjoint decomposition
%%%		Action: incompatibility information is initialized

initializeDisjointDecomposition([]).
initializeDisjointDecomposition([C|DC]):-
		initializeListOfDisjointClassesWith(C,DC),
		initializeDisjointDecomposition(DC).

%%% initializeSubrelationsOf(in R)
%%%		R: relation
%%%		Action: subrelations of R are initialized

initializeSubrelationsOf(R):-
		findall((F,SR),axiom(_,F,_,_,_,'subrelation'(SR,R)),LSR),
		initializeListOfSubrelationsOf(R,LSR).

%%% initializeListOfSubrelationsOf(in R, in LSR)
%%%		R: relation
%%%		LSR: a list of pairs (F,SR) where
%%%			F: ontology file where SR occurs in
%%%			SR: subrelation of R
%%%		Action: subrelations of R are initialized

initializeListOfSubrelationsOf(_,[]).
initializeListOfSubrelationsOf(R,[(F,SR)|LSR]):-
		assertWithoutRepetition('&subrelationOf'(SR,R),_),
		assertWithoutRepetition('&ontologyOf'(SR,F),_),
		initializeSubrelationInformation(SR),
		initializeListOfSubrelationsOf(R,LSR).

%%% initializeSubAttributesOf(in A)
%%%		A: attribute
%%%		Action: subAttritutes of A are initialized

initializeSubAttributesOf(A):-
		findall((F,SA),axiom(_,F,_,_,_,'subAttribute'(SA,A)),LSA),
		initializeListOfSubAttributesOf(A,LSA).

%%% initializeListOfSubAttributesOf(in A, in LSA)
%%%		A: attribute
%%%		LSA: a list of pairs (F,SA) where
%%%			F: ontology file where SA occurs in
%%%			SA: subAttribute of A
%%%		Action: subAttributes of A are initialized

initializeListOfSubAttributesOf(_,[]).
initializeListOfSubAttributesOf(A,[(F,SA)|LSA]):-
		assertWithoutRepetition('&subAttributeOf'(SA,A),_),
		assertWithoutRepetition('&ontologyOf'(SA,F),_),
		initializeSubAttributeInformation(SA),
		initializeListOfSubAttributesOf(A,LSA).

%%% assertWithoutRepetition(in A, out N)
%%%		A: an atom
%%%		N:
%%%			0	if A IS a new atom
%%%			1	if A is NOT a new atom
%%%		Action: internal information is stored using dynamic predicates

assertWithoutRepetition(A,1):-
		A,
		!.
assertWithoutRepetition(A,0):-
		assertz(A).


% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Consulting auxiliary information										% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

% % All the predicates to consult internal information

% %			- getTopClass/1
% %			- getSubclassOf/2
% %			- getSuperClassOf/2
% %			- getInstanceOf/2
% %			- getClassOf/2
% %			- getSubrelationOf/2
% %			- getSuperRelationOf/2
% %			- getSubAttributeOf/2
% %			- getSuperAttributeOf/2

% %			- getDirectSubclassOf/2
% %			- getDirectSuperClassOf/2
% %			- getDirectClassOf/2
% %			- getDirectSuperRelationOf/2
% %			- getDirectSuperAttributeOf/2
% %			- getDirectSuperConceptOf/2

% %			- getAllTopClasses/1
% %			- getAllSubclassesOf/2
% %			- getAllSuperClassesOf/2
% %			- getAllInstancesOf/2
% %			- getAllClassesOf/2
% %			- getAllSubrelationsOf/2
% %			- getAllSuperRelationsOf/2
% %			- getAllSubAttributesOf/2
% %			- getAllSuperAttributesOf/2

% %			- getAllDirectSubclassesOf/2
% %			- getAllDirectSuperClassesOf/2
% %			- getAllDirectClassesOf/2
% %			- getAllDirectSuperRelationsOf/2
% %			- getAllDirectSuperAttributesOf/2
% %			- getAllDirectSuperConceptsOf/2

% %			- isTopClass/1
% %			- isInstanceOf/2
% %			- isSubclassOf/2
% %			- areDisjoint/2
% %			- isSubrelationOf/2
% %			- isSubAttributeOf/2

% %			- isAdimenSUMO/1

% %			- isClass/1
% %			- isObject/1
% %			- isRelation/1
% %			- isAttribute/1

% %			- isObjectRelation/1
% %			- isObjectAttribute/1
% %			- isClassRelation/1
% %			- isClassAttribute/1

% %			- getAllClasses/1
% %			- getAllObjects/1
% %			- getAllRelations/1
% %			- getAllAttributes/1

% %			- getTypeOfConcept/2

%%% getTopClass(out C)
%%%		C: top class

getTopClass(C):-
		'&topClass'(C).

%%% getSubclassOf(in C, out SC)
%%%		C: class
%%%		SC: a subclass of C

getSubclassOf(C,SC):-
		nonvar(C),
		!,
		(
		 (
		  C = SC
		 );
		 (
		  '&subclassOf'(SSC,C),
		  getSubclassOf(SSC,SC)
		 )
		).
getSubclassOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getSubclassOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getSuperClassOf(in C, out SC)
%%%		C: class
%%%		SC: a super-class of C

getSuperClassOf(C,SC):-
		nonvar(C),
		!,
		(
		 (
		  C = SC
		 );
		 (
		  '&subclassOf'(C,SSC),
		  getSuperClassOf(SSC,SC)
		 )
		).
getSuperClassOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getSuperClassOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getInstanceOf(in C, out O)
%%%		C: class
%%%		O: an instance of C

getInstanceOf(C,O):-
		nonvar(C),
		!,
		getSubclassOf(C,SC),
		'&instanceOf'(X,SC),
		(
		 (
		  O = X
		 );
		 (
		  getSubAttributeOf(X,O),
		  O \= X
		 );
		 (
		  isInstanceOf(X,'InheritableRelation'),
		  getSubrelationOf(X,O),
		  O \= X
		 )
		).
getInstanceOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getInstanceOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getClassOf(in O, out C)
%%%		O: object
%%%		C: a super-class of O

getClassOf(O,C):-
		nonvar(O),
		!,
		(
		 (
		  '&instanceOf'(O,SC),
		  getSuperClassOf(SC,C)
		 );
		 (
		  getSuperAttributeOf(O,SA),
		  O \= SA,
		  getClassOf(SA,C)
		 );
		 (
		  isInstanceOf(C,'InheritableRelation'),
		  getSuperRelationOf(O,SR),
		  O \= SR,
		  getClassOf(SR,C)
		 )
		).
getClassOf(O,_):-
		var(O),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getClassOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getSubrelationOf(in R, out SR)
%%%		R: relation
%%%		SR: a subrelation of R

getSubrelationOf(R,SR):-
		nonvar(R),
		!,
		(
		 (
		  R = SR
		 );
		 (
		  '&subrelationOf'(SSR,R),
		  getSubrelationOf(SSR,SR)
		 )
		).
getSubrelationOf(R,_):-
		var(R),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getSubrelationOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getSuperRelationOf(in R, out SR)
%%%		R: relation
%%%		SR: a super-relation of R

getSuperRelationOf(R,SR):-
		nonvar(R),
		!,
		(
		 (
		  R = SR
		 );
		 (
		  '&subrelationOf'(R,SSR),
		  getSuperRelationOf(SSR,SR)
		 )
		).
getSuperRelationOf(R,_):-
		var(R),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getSuperRelationOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getSubAttributeOf(in A, out SA)
%%%		A: attribute
%%%		SA: a subAttribute of A

getSubAttributeOf(A,SA):-
		nonvar(A),
		!,
		(
		 (
		  A = SA
		 );
		 (
		  '&subAttributeOf'(SSA,A),
		  getSubAttributeOf(SSA,SA)
		 )
		).
getSubAttributeOf(A,_):-
		var(A),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getSubAttributeOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getSuperAttributeOf(in A, out SA)
%%%		A: attribute
%%%		SA: a super-attribute of A

getSuperAttributeOf(A,SA):-
		nonvar(A),
		!,
		(
		 (
		  A = SA
		 );
		 (
		  '&subAttributeOf'(A,SSA),
		  getSuperAttributeOf(SSA,SA)
		 )
		).
getSuperAttributeOf(A,_):-
		var(A),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getSuperAttributeOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getDirectSubclassOf(in C, out SC)
%%%		C: class
%%%		SC: a direct subclass of C

getDirectSubclassOf(C,SC):-
		nonvar(C),
		!,
		'&subclassOf'(SC,C).
getDirectSubclassOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getDirectSubclassOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getDirectSuperClassOf(in C, out SC)
%%%		C: class
%%%		SC: a direct super-class of C

getDirectSuperClassOf(C,SC):-
		nonvar(C),
		!,
		'&subclassOf'(C,SC).
getDirectSuperClassOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getDirectSuperClassOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getDirectClassOf(in O, out C)
%%%		O: object
%%%		C: a direct super-class of O

getDirectClassOf(O,C):-
		nonvar(O),
		!,
		'&instanceOf'(O,C).
getDirectClassOf(O,_):-
		var(O),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getDirectClassOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getDirectSuperRelationOf(in R, out SR)
%%%		R: relation
%%%		SR: a direct super-relation of R

getDirectSuperRelationOf(R,SR):-
		nonvar(R),
		!,
		'&subrelationOf'(R,SR).
getDirectSuperRelationOf(R,_):-
		var(R),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getDirectSuperRelationOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getDirectSuperAttributeOf(in A, out SA)
%%%		A: attribute
%%%		SA: a direct super-attribute of A

getDirectSuperAttributeOf(A,SA):-
		nonvar(A),
		!,
		'&subAttributeOf'(A,SA).
getDirectSuperAttributeOf(A,_):-
		var(A),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getDirectSuperAttributeOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getDirectSuperConceptOf(in C, out SC)
%%%		C: concept
%%%		SC: a direct super-concept of C

getDirectSuperConceptOf(C,SC):-
		nonvar(C),
		!,
		(
		 (
		  '&subclassOf'(C,SC)
		 );
		 (
		  '&instanceOf'(C,SC)
		 );
		 (
		  '&subrelationOf'(C,SC)
		 );
		 (
		  '&subAttributeOf'(C,SC)
		 )
		).
getDirectSuperConceptOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getDirectSuperConceptOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllTopClasses(out LC)
%%%		LC: the list of top classes

getAllTopClasses(LC):-
		findall(C,getTopClass(C),LC).

%%% getAllSubclassesOf(in C, out LC)
%%%		C: class
%%%		LC: the list of subclasses of C

getAllSubclassesOf(C,LC):-
		nonvar(C),
		!,
		findall(SC,getSubclassOf(C,SC),LC).
getAllSubclassesOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllSubclassesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllSuperClassesOf(in C, out LC)
%%%		C: class
%%%		LC: the list of super-classes of C

getAllSuperClassesOf(C,LC):-
		nonvar(C),
		!,
		findall(SC,getSuperClassOf(C,SC),LC).
getAllSuperClassesOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllSuperClassesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllInstancesOf(in C, out LO)
%%%		C: class
%%%		LO: the list of instances of C

getAllInstancesOf(C,LO):-
		nonvar(C),
		!,
		findall(O,getInstanceOf(C,O),LO).
getAllInstancesOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllInstancesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllClassesOf(in O, out LC)
%%%		O: object
%%%		LC: the list of classes of C

getAllClassesOf(O,LC):-
		nonvar(O),
		!,
		findall(C,getClassOf(O,C),LC).
getAllClassesOf(O,_):-
		var(O),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllClassesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllSubrelationsOf(in R, out LR)
%%%		R: relation
%%%		LR: the list of relations of R

getAllSubrelationsOf(R,LR):-
		nonvar(R),
		!,
		findall(SR,getSubrelationOf(R,SR),LR).
getAllSubrelationsOf(R,_):-
		var(R),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllSubrelationsOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllSuperRelationsOf(in R, out LR)
%%%		R: relation
%%%		LR: the list of subrelations of R

getAllSuperRelationsOf(R,LR):-
		nonvar(R),
		!,
		findall(SR,getSuperRelationOf(R,SR),LR).
getAllSuperRelationsOf(R,_):-
		var(R),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllSuperRelationsOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllSubAttributesOf(in A, out LA)
%%%		A: attribute
%%%		LA: the list of subAttributes of A

getAllSubAttributesOf(A,LA):-
		nonvar(A),
		!,
		findall(SA,getSubAttributeOf(A,SA),LA).
getAllSubAttributesOf(A,_):-
		var(A),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllSubAttributesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllSubAttributesOf(in A, out LA)
%%%		A: attribute
%%%		LA: the list of subAttributes of A

getAllSubAttributesOf(A,LA):-
		nonvar(A),
		!,
		findall(SA,getSubAttributeOf(A,SA),LA).
getAllSubAttributesOf(A,_):-
		var(A),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllSubAttributesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllDirectSubclassesOf(in C, out LC)
%%%		C: class
%%%		LC: the list of direct subclasses of C

getAllDirectSubclassesOf(C,LC):-
		nonvar(C),
		!,
		findall(SC,getDirectSubclassOf(C,SC),LC).
getAllDirectSubclassesOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllDirectSubclassesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllDirectSuperClassesOf(in C, out LC)
%%%		C: class
%%%		LC: the list of direct super-classes of C

getAllDirectSuperClassesOf(C,LC):-
		nonvar(C),
		!,
		findall(SC,getDirectSuperClassOf(C,SC),LC).
getAllDirectSuperClassesOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllDirectSuperClassesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllDirectClassesOf(in O, out LC)
%%%		O: object
%%%		LC: the list of direct classes of C

getAllDirectClassesOf(O,LC):-
		nonvar(O),
		!,
		findall(C,getDirectClassOf(O,C),LC).
getAllDirectClassesOf(O,_):-
		var(O),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllDirectClassesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllDirectSuperRelationsOf(in R, out LR)
%%%		R: relation
%%%		LR: the list of direct super-relations of R

getAllDirectSuperRelationsOf(R,LR):-
		nonvar(R),
		!,
		findall(SR,getDirectSuperRelationOf(R,SR),LR).
getAllDirectSuperRelationsOf(R,_):-
		var(R),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllDirectSuperRelationsOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllDirectSuperAttributesOf(in A, out LA)
%%%		A: attribute
%%%		LA: the list of direct super-attributes of A

getAllDirectSuperAttributesOf(A,LA):-
		nonvar(A),
		!,
		findall(SA,getDirectSuperAttributeOf(A,SA),LA).
getAllDirectSuperAttributesOf(A,_):-
		var(A),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllDirectSuperAttributesOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllDirectSuperConceptsOf(in C, out LC)
%%%		C: concept
%%%		LC: the list of direct super-concepts of C

getAllDirectSuperConceptsOf(C,LC):-
		nonvar(C),
		!,
		findall(SC,getDirectSuperConceptOf(C,SC),LC).
getAllDirectSuperConceptsOf(C,_):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of getAllDirectSuperConceptsOf: variable as first argument.'),
		print_to_screen('\n'),
		fail.

%%% isTopClass(in C)
%%%		C: class
%%%		Action:
%%%				success	if C IS a top class
%%%				failure	if C is NOt a top class

isTopClass(C):-
		nonvar(C),
		getTopClass(C),
		!.
isTopClass(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isTopClass: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isInstanceOf(in O, in C)
%%%		O: object
%%%		C: class
%%%		Action:
%%%			success		if O IS instance of C
%%%			fail		if O is NOT instance of C

isInstanceOf(O,C):-
		nonvar(O),
		nonvar(C),
		O == C,
		!,
		fail.
isInstanceOf(O,C):-
		nonvar(O),
		nonvar(C),
		getClassOf(O,C),
		!.
isInstanceOf(O,C):-
		(
		 (
		  var(O)
		 );
		 (
		  var(C)
		 )
		),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isInstanceOf: variable(s) as argument(s).'),
		print_to_screen('\n'),
		fail.

%%% isSubclassOf(in SC, in C)
%%%		SC,C: classes
%%%		Action:
%%%			success		if SC IS subclass of C
%%%			fail		if SC is NOT subclass of C

isSubclassOf(SC,C):-
		nonvar(SC),
		nonvar(C),
		getSuperClassOf(SC,C),
		!.
isSubclassOf(SC,C):-
		(
		 (
		  var(SC)
		 );
		 (
		  var(C)
		 )
		),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isSubclassOf: variable(s) as argument(s).'),
		print_to_screen('\n'),
		fail.

%%% areDisjoint(in C1, in C2)
%%%		C1, C2: classes
%%%		Action:
%%%			success		if C1 and C2 ARE disjoint
%%%			fail		if C1 and C2 are NOT disjoint

areDisjoint(C1,C2):-
		nonvar(C1),
		nonvar(C2),
		getSuperClassOf(C1,SC1),
		getSuperClassOf(C2,SC2),
		(
		 (
		  '&disjoint'(SC1,SC2)
		 );
		 (
		  '&disjoint'(SC2,SC1)
		 )
		),
		!.
areDisjoint(C1,C2):-
		(
		 (
		  var(C1)
		 );
		 (
		  var(C2)
		 )
		),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of areDisjoint: variable(s) as argument(s).'),
		print_to_screen('\n'),
		fail.

%%% isSubrelationOf(in SR, in R)
%%%		SR,R: relations
%%%		Action:
%%%			success		if SR IS subrelation of R
%%%			fail		if SR is NOT subrelation of R

isSubrelationOf(SR,R):-
		nonvar(SR),
		nonvar(R),
		getSubrelationOf(R,SR),
		!.
isSubrelationOf(SR,R):-
		(
		 (
		  var(SR)
		 );
		 (
		  var(R)
		 )
		),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isSubrelationOf: variable(s) as argument(s).'),
		print_to_screen('\n'),
		fail.

%%% isSubAttributeOf(in SA, in A)
%%%		SA,A: attributes
%%%		Action:
%%%			success		if SA IS subAttribute of A
%%%			fail		if SA is NOT subAttribute of A

isSubAttributeOf(SA,A):-
		nonvar(SA),
		nonvar(A),
		getSubAttributeOf(A,SA),
		!.
isSubAttributeOf(SA,A):-
		(
		 (
		  var(SA)
		 );
		 (
		  var(A)
		 )
		),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isSubAttributeOf: variable(s) as argument(s).'),
		print_to_screen('\n'),
		fail.

%%% isSUMO(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C BELONGS to SUMO
%%%				failure	if C does NOT belong to SUMO

isSUMO(C):-
		nonvar(C),
		'&ontologyOf'(C,_),
		!.
isSUMO(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isSUMO: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isAdimenSUMO(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C BELONGS to AdimenSUMO
%%%				failure	if C does NOT belong to AdimenSUMO

isAdimenSUMO(C):-
		nonvar(C),
		'&ontologyOf'(C,O),
		getAdimenSUMOList(LO),
		member(O,LO),
		!.
isAdimenSUMO(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isAdimenSUMO: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isClass(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS a class
%%%				failure	if C is NOT a class

isClass(C):-
		nonvar(C),
		isSubclassOf(C,'Entity'),
		\+ isRelation(C),
		\+ isAttribute(C),
		!.
isClass(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isClass: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isObject(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS an object
%%%				failure	if C is NOT an object

isObject(C):-
		nonvar(C),
		isInstanceOf(C,'Entity'),
		\+ isRelation(C),
		\+ isAttribute(C),
		!.
isObject(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isObject: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isRelation(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS a relation
%%%				failure	if C is NOT a relation

isRelation(C):-
		nonvar(C),
		(
		 (
		  isInstanceOf(C,'Relation')
		 );
		 (
		  isSubclassOf(C,'Relation')
		 )
		),
		!.
isRelation(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isRelation: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isAttribute(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS an attribute
%%%				failure	if C is NOT an attribute

isAttribute(C):-
		nonvar(C),
		(
		 (
		  isInstanceOf(C,'Attribute')
		 );
		 (
		  isSubclassOf(C,'Attribute')
		 )
		),
		!.
isAttribute(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isAttribute: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isObjectRelation(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS an object relation
%%%				failure	if C is NOT an object relation

isObjectRelation(C):-
		nonvar(C),
		isInstanceOf(C,'Relation'),
		!.
isObjectRelation(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isObjectRelation: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isObjectAttribute(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS an object attribute
%%%				failure	if C is NOT an object attribute

isObjectAttribute(C):-
		nonvar(C),
		isInstanceOf(C,'Attribute'),
		!.
isObjectAttribute(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isObjectAttribute: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isClassRelation(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS a class relation
%%%				failure	if C is NOT a class relation

isClassRelation(C):-
		nonvar(C),
		isSubclassOf(C,'Relation'),
		!.
isClassRelation(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isClassRelation: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% isClassAttribute(in C)
%%%		C: concept
%%%		Action:
%%%				success	if C IS a class attribute
%%%				failure	if C is NOT a class attribute

isClassAttribute(C):-
		nonvar(C),
		isSubclassOf(C,'Attribute'),
		!.
isClassAttribute(C):-
		var(C),
		print_to_screen('\n'),
		print_to_screen('Incorrect use of isClassAttribute: variable as argument.'),
		print_to_screen('\n'),
		fail.

%%% getAllClasses(out LC)
%%%		LC: list of all classes defined in SUMO

getAllClasses(LC):-
		findall(C,'&initializedClass'(C),LC).

%%% getAllObjects(out LO)
%%%		LO: list of all objects defined in SUMO

getAllObjects(LO):-
		findall(O,'&initializedObject'(O),LO).

%%% getAllRelations(out LR)
%%%		LR: list of all relations defined in SUMO

getAllRelations(LR):-
		findall(R,'&initializedSubrelation'(R),LR).

%%% getAllAttributes(out LA)
%%%		LA: list of all attributes defined in SUMO

getAllAttributes(LA):-
		findall(A,'&initializedSubAttribute'(A),LA).

%%% isAdimenSUMOClass(in C)
%%%		C: AdimenSUMO concept
%%%		Action:
%%%				success	if C IS an AdimenSUMO class
%%%				failure	if C is NOT AdimenSUMO class

isAdimenSUMOClass(C):-
		isSubclassOf(C,'Entity').

%%% isAdimenSUMOObject(in O)
%%%		C: AdimenSUMO concept
%%%		Action:
%%%				success	if C IS an AdimenSUMO object
%%%				failure	if C is NOT AdimenSUMO object

isAdimenSUMOObject(C):-
		isInstanceOf(C,'Entity').

%%% obtainAdimenSUMOHierarchy
%%%		Action: the hierarchy of AdimenSUMO is written in a file

obtainAdimenSUMOHierarchy:-
		open('Ontologies/AdimenSUMO.txt',write,S),
		getAllTopClasses(LC),
		obtainAdimenSUMOHierarchyListOfTopClasses(S,LC),
		close(S).

%%% obtainAdimenSUMOHierarchyListOfTopClasses(in S, in LC)
%%%		S: output stream
%%%		LC: list of classes
%%%		Action: the hierarchy of classes of AdimenSUMO is written in a file

obtainAdimenSUMOHierarchyListOfTopClasses(_,[]).
obtainAdimenSUMOHierarchyListOfTopClasses(S,[C|LC]):-
		isAdimenSUMO(C),
		!,
		getAllDirectSubclassesOf(C,LSC),
		obtainAdimenSUMOHierarchyListOfSubclasses(S,LSC,C),
		obtainAdimenSUMOHierarchyListOfTopClasses(S,LC).

obtainAdimenSUMOHierarchyListOfTopClasses(S,[_|LC]):-
		obtainAdimenSUMOHierarchyListOfTopClasses(S,LC).

%%% obtainAdimenSUMOHierarchyListOfSubclasses(in S, in LSC, in SC)
%%%		S: output stream
%%%		LC: list of classes
%%%		Action: the hierarchy of classes of AdimenSUMO is written in a file

obtainAdimenSUMOHierarchyListOfSubclasses(_,[],_).
obtainAdimenSUMOHierarchyListOfSubclasses(S,[SC|LSC],C):-
		isAdimenSUMO(SC),
		!,
		print_to_file(S,SC),
		print_to_file(S,' subclass-of '),
		print_to_file(S,C),
		print_to_file(S,'\n'),
		getAllDirectSubclassesOf(SC,LSSC),
		obtainAdimenSUMOHierarchyListOfSubclasses(S,LSSC,SC),
		obtainAdimenSUMOHierarchyListOfSubclasses(S,LSC,C).
obtainAdimenSUMOHierarchyListOfSubclasses(S,[_|LSC],C):-
		obtainAdimenSUMOHierarchyListOfSubclasses(S,LSC,C).

%%% saveAdimenSUMOHierarchy
%%%		Action: AdimenSUMO hierarchy is written in a file

saveAdimenSUMOHierarchy:-
		open('Mappings/AdimenSUMO.pl',write,S),
		findall(('&topClass',C),'&topClass'(C),LTC),
		writeListOfAdimenSUMOAxioms(S,LTC),
		findall(('&subclassOf',C1,C2),'&subclassOf'(C1,C2),LSC),
		writeListOfAdimenSUMOAxioms(S,LSC),
		findall(('&instanceOf',O,C),'&instanceOf'(O,C),LO),
		writeListOfAdimenSUMOAxioms(S,LO),
		findall(('&subrelationOf',R1,R2),'&subrelationOf'(R1,R2),LSR),
		writeListOfAdimenSUMOAxioms(S,LSR),
		findall(('&subAttributeOf',A1,A2),'&subAttributeOf'(A1,A2),LSA),
		writeListOfAdimenSUMOAxioms(S,LSA),
		close(S).

%%% writeListOfAdimenSUMOAxioms(in S, in LA)
%%%		S: output stream
%%%		LA: list of AdimenSUMO axioms
%%%		Action: the list of AdimenSUMO axioms LA is written in S

writeListOfAdimenSUMOAxioms(_,[]).
writeListOfAdimenSUMOAxioms(S,[(R,C1,C2)|LR]):-
		!,
		print_to_file(S,'\''),
		print_to_file(S,R),
		print_to_file(S,'\'(\''),
		print_to_file(S,C1),
		print_to_file(S,'\',\''),
		print_to_file(S,C2),
		print_to_file(S,'\').\n'),
		writeListOfAdimenSUMOAxioms(S,LR).
writeListOfAdimenSUMOAxioms(S,[(R,C)|LR]):-
		print_to_file(S,'\''),
		print_to_file(S,R),
		print_to_file(S,'\'(\''),
		print_to_file(S,C),
		print_to_file(S,'\').\n'),
		writeListOfAdimenSUMOAxioms(S,LR).

%%% getTypeOfConcept(in C, out T)
%%%		C: concept
%%%		T: type of concept

getTypeOfConcept(C,'relation'):-
		isObjectRelation(C),
		!.
getTypeOfConcept(C,'Relation'):-
		isClassRelation(C),
		!.
getTypeOfConcept(C,'attribute'):-
		isObjectAttribute(C),
		!.
getTypeOfConcept(C,'Attribute'):-
		isClassAttribute(C),
		!.
getTypeOfConcept(C,'object'):-
		isObject(C),
		!.
getTypeOfConcept(C,'class'):-
		isClass(C),
		!.
